import pygame
import sys
sys.path.append('../')
import PIC32Interface
import time
import os
import RPi.GPIO as GPIO

#display on piTFT 
os.putenv('SDL_VIDEORIVER','fbcon')
os.putenv('SDL_FBDEV','/dev/fb1')

#initialize GPIO

GPIO.setmode(GPIO.BCM)

GPIO.setup(4, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(17, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(22, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(23, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(27, GPIO.IN, pull_up_down=GPIO.PUD_UP)

#callback functions

#time Up
def GPIO17_callback(channel):
   global time_Index
   time_Index= time_Index+1
   if (time_Index> 5):
      time_Index= 5
      
   print(time_Index)
   
#time Down
def GPIO22_callback(channel):
   global time_Index

   time_Index= time_Index-1

   if (time_Index<0):

      time_Index= 0

   print(time_Index)
   

#volt Up
def GPIO23_callback(channel):
   global volt_Index
   volt_Index = volt_Index+1

   if (volt_Index> 3):

      volt_Index= 3

   print(volt_Index)
   
#time Down
def GPIO27_callback(channel):
   global volt_Index
   volt_Index = volt_Index-1

   if (volt_Index< 0):

      volt_Index= 0

   print(volt_Index)



#add event detection for the GPIO
GPIO.add_event_detect(17, GPIO.FALLING, callback=GPIO17_callback, bouncetime=300)
GPIO.add_event_detect(22, GPIO.FALLING, callback=GPIO22_callback, bouncetime=300)
GPIO.add_event_detect(23, GPIO.FALLING, callback=GPIO23_callback, bouncetime=300)
GPIO.add_event_detect(27, GPIO.FALLING, callback=GPIO27_callback, bouncetime=300)


# Define some colors
BLACK = (0, 0, 0)
WHITE = (255, 255, 255)
GREEN = (0, 255, 0)
RED = (255, 0, 0)
PURPLE = (238,130,238)

top = 0
bottom = 239
left = 0
right = 319

tpd_list = ['200us','500us','  1ms','  2ms',' 10ms','20ms']
vpd_list = ['   2V','   1V','500mV','250mV']
vpd_list_num = [2,1,0.5,0.25]
sample_list  = [160,64,32,16,6,3]

time_Index = 0
volt_Index = 1

x_axis = []
lines = []

# Initialize pygame
pygame.init()

# Set the HEIGHT and WIDTH of the screen
WINDOW_SIZE = [320, 240]
screen = pygame.display.set_mode(WINDOW_SIZE)

# Set title of screen
pygame.display.set_caption("Pic&Pi Scope")

# Loop until the user clicks the close button.
done = False

# Used to manage how fast the screen updates
clock = pygame.time.Clock()

data_len = 255
for i in range(0,data_len):
    lines.append([0,0])
    x_axis.append(i*1.25)



# -------- Main Program Loop -----------
while not done:
    
    if(not GPIO.input(4)) : break;
    for event in pygame.event.get():  # User did something
        if event.type == pygame.QUIT:  # If user clicked close
            done = True  # Flag that we are done so we exit this loop
            
    #===========Pic & Pi set up sample frquence and read Data=======
    PIC32Interface._SetSampleFreq_( sample_list[time_Index],512)
    PIC32Interface._StartADC_( 0,3 )
    time.sleep(1)
    while((PIC32Interface.CheckBufferStatus() != 0x0F) ):
        time.sleep(1)
    ReadData=PIC32Interface.ReadBuffer2(3,256)

    plotdata=[3.3*(float(x)/float(1024)) for x in ReadData]
    

    # Set the screen background
    screen.fill(BLACK)

    scope_text = "CH1 = %s   M = %s " % (vpd_list[volt_Index], tpd_list[time_Index])

    font = pygame.font.Font('freesansbold.ttf', 20)

    scope_info = font.render(scope_text,True,PURPLE)

    writePlace = scope_info.get_rect(center = (160,220))

    pygame.draw.circle(screen, RED, (0,200), 5)

    #colums
    for i in range(0,8):
        pygame.draw.line(screen,WHITE,[i*40,top],[i*40,bottom],1)
    #rows
    for i in range(0,6):
        pygame.draw.line(screen,WHITE,[left,i*40],[right,i*40],1)

    #=======plot scope data==========

    for i in range(0,data_len):
        val = 240 - plotdata[i]/vpd_list_num[volt_Index]*40 -40
        lines[i] = [x_axis[i],val]
        
    #print lines

    pygame.draw.lines(screen,GREEN,0,lines,2)

    #======end scope data============

    screen.blit(scope_info,writePlace)

    # Limit to 100 frames per second
    clock.tick(100)

    # Go ahead and update the screen with what we've drawn.
    pygame.display.flip()

# avoid the program hang upon exit
pygame.quit()
